# These are only needed, if you are using custom conversion rates.
# If you are valuating everything in terms of work hour value,
# then you don't need to enter this information.

# Currency value ( get one at openexchangerates.org )
OPENEXCHANGE_APP_ID = ''
OPENEXCHANGE_BASE_CURRENCY = 'USD'
OPENEXCHANGE_URL = 'https://openexchangerates.org/api/latest.json'

# [Optional] Hour value ( get one at api.stlouisfed.org )
FRED_KEY = ''
FRED_SERIES = 'CES0500000003'
DEFAULT_HOUR_VALUE_IN_USD = 25.0

